import unittest

from automon.helpers.sanitation import Sanitation


class SanitationTest(unittest.TestCase):
    def test_Sanitation(self):
        self.assertTrue(Sanitation)
        self.assertTrue(Sanitation.strip_quotes)
        self.assertTrue(Sanitation.strip_spaces)
        self.assertTrue(Sanitation.strip_spaces_from_list)
        self.assertTrue(Sanitation.safe_string)
        self.assertTrue(Sanitation.dedup)
        self.assertTrue(Sanitation.list_from_string)

        self.assertRaises(TypeError, Sanitation.strip_quotes)
        self.assertRaises(TypeError, Sanitation.strip_spaces)
        self.assertRaises(TypeError, Sanitation.strip_spaces_from_list)
        self.assertRaises(TypeError, Sanitation.safe_string)
        self.assertRaises(TypeError, Sanitation.dedup)
        self.assertRaises(TypeError, Sanitation.list_from_string)

        self.assertEqual(Sanitation.strip_quotes('test'), 'test')
        self.assertEqual(Sanitation.strip_spaces(' t e s t '), 't e s t')
        self.assertEqual(Sanitation.strip_spaces(' t e s t '), 't e s t')
        self.assertEqual(Sanitation.strip_spaces_from_list(
            [' t e s t ']), ['t e s t'])
        self.assertEqual(Sanitation.safe_string(
            'a;skdfjAS*&D)(&!H!:@JEN'), 'a_skdfjAS__D____H___JEN')
        self.assertEqual(Sanitation.dedup(
            [1, 1, 'AAAAA', 'AAAAA', 'AA', 555, 555, 5555, 6666, 7, 8, 33, 1, 1232, 499124, 'a']),
            [1, 'AAAAA', 'AA', 555, 5555, 6666, 7, 8, 33, 1232, 499124, 'a'])

        self.assertEqual(Sanitation.list_from_string(
            'a,d,a,3,1,5,g,2,,a,4,1,,h,4,1,k,z'),
            ['a', 'd', 'a', '3', '1', '5', 'g', '2', '', 'a', '4', '1', '', 'h', '4', '1', 'k', 'z']
        )
        self.assertEqual(Sanitation.list_from_string(
            'a d a 3 1 5 g 2  a 4 1  h 4 1 k z'),
            ['a', 'd', 'a', '3', '1', '5', 'g', '2', '', 'a', '4', '1', '', 'h', '4', '1', 'k', 'z']
        )
        self.assertEqual(Sanitation.list_from_string(
            ' ada315g2a41h41kz '),
            ['ada315g2a41h41kz']
        )


if __name__ == '__main__':
    unittest.main()



from automon.log import Logging
from automon.integrations.elasticsearch.client import ElasticsearchClient

log = Logging(__name__, Logging.DEBUG)


class Cleanup:

    def __init__(self):
        self.client = ElasticsearchClient()

    def get_indices(self):
        return self.client.get_indices()

    def search_indices(self, index_pattern):
        return self.client.search_indices(index_pattern)

    # def delete_indices(self, index_pattern):
    #     return self.client.delete_indices(index_pattern)



import os

from automon.log import Logging
from automon.helpers.sanitation import Sanitation as S

log = Logging(name='Neo4jConfig', level=Logging.DEBUG)


class Neo4jConfig:
    def __init__(self, user: str = None,
                 password: str = None,
                 hosts: str = None,
                 encrypted: bool = None,
                 trust: bool = None):
        """Neo4j config
        """

        self.NEO4J_USER = user or os.getenv('NEO4J_USER') or ''
        self.NEO4J_PASSWORD = password or os.getenv('NEO4J_PASSWORD') or ''
        self.NEO4J_HOST = hosts or os.getenv('NEO4J_HOST') or ''

        self.encrypted = encrypted
        self.trust = trust

        if not self.NEO4J_USER: log.warn(f'missing NEO4J_USER')
        if not self.NEO4J_PASSWORD: log.warn(f'missing NEO4J_PASSWORD')
        if not self.NEO4J_HOST: log.warn(f'missing NEO4J_HOST')

    def __repr__(self):
        return f'{self.__dict__}'



import json
import requests

from automon.log import Logging
from .config import RequestsConfig

log = Logging(name='RequestsClient', level=Logging.DEBUG)


class RequestsClient(object):
    def __init__(self, url: str = None, data: dict = None, headers: dict = None,
                 config: RequestsConfig = None):
        """Wrapper for requests library"""

        self.config = config or RequestsConfig()

        self.url = url
        self.data = data
        self.headers = headers
        self.results = None
        self.requests = requests

        if url:
            self.url = url
            self.get(url=self.url, data=self.data, headers=self.headers)

    def __repr__(self):
        return f'{self.__dict__}'

    def _log_result(self):
        if self.results.status_code == 200:
            msg = f'{self.results.status_code} ' \
                  f'{self.results.url} ' \
                  f'{round(len(self.results.content) / 1024, 2)} KB'
            return log.debug(msg)

        msg = f'{self.results.status_code} ' \
              f'{self.results.url} ' \
              f'{round(len(self.results.content) / 1024, 2)} KB ' \
              f'{self.results.content}'
        return log.error(msg, raise_exception=False)

    def _params(self, url, data, headers):
        if url is None:
            url = self.url

        if data is None:
            data = self.data

        if headers is None:
            headers = self.headers

        return url, data, headers

    @property
    def content(self):
        if self.results is not None:
            return self.results.content

    def delete(self,
               url: str = None,
               data: dict = None,
               headers: dict = None, **kwargs) -> bool:
        """requests.delete"""

        url, data, headers = self._params(url, data, headers)

        try:
            self.results = requests.delete(url=url, data=data, headers=headers, **kwargs)
            self._log_result()
            return True
        except Exception as e:
            log.error(f'delete failed. {e}', enable_traceback=False)
        return False

    def get(self,
            url: str = None,
            data: dict = None,
            headers: dict = None, **kwargs) -> bool:
        """requests.get"""

        url, data, headers = self._params(url, data, headers)

        try:
            self.results = requests.get(url=url, data=data, headers=headers, **kwargs)
            self._log_result()
            return True
        except Exception as e:
            log.error(f'get failed. {e}', enable_traceback=False)
        return False

    def patch(self,
              url: str = None,
              data: dict = None,
              headers: dict = None, **kwargs) -> bool:
        """requests.patch"""

        url, data, headers = self._params(url, data, headers)

        try:
            self.results = requests.patch(url=url, data=data, headers=headers, **kwargs)
            self._log_result()
            return True
        except Exception as e:
            log.error(f'patch failed. {e}', enable_traceback=False)
        return False

    def post(self,
             url: str = None,
             data: dict = None,
             headers: dict = None, **kwargs) -> bool:
        """requests.post"""

        url, data, headers = self._params(url, data, headers)

        try:
            self.results = requests.post(url=url, data=data, headers=headers, **kwargs)
            self._log_result()
            return True
        except Exception as e:
            log.error(f'post failed. {e}', enable_traceback=False)
        return False

    def put(self,
            url: str = None,
            data: dict = None,
            headers: dict = None, **kwargs) -> bool:
        """requests.put"""

        url, data, headers = self._params(url, data, headers)

        try:
            self.results = requests.put(url=url, data=data, headers=headers, **kwargs)
            self._log_result()
            return True
        except Exception as e:
            log.error(f'put failed. {e}', enable_traceback=False)
        return False

    def to_dict(self):
        if self.results is not None:
            return json.loads(self.results.content)


class Requests(RequestsClient):
    pass



import unittest

from automon.helpers.sleeper import Sleeper


class SleeperTest(unittest.TestCase):
    def test_Sleeper(self):
        self.assertTrue(Sleeper)
        self.assertTrue(Sleeper.seconds)
        self.assertTrue(Sleeper.minute)
        self.assertTrue(Sleeper.within_a_minute)
        self.assertTrue(Sleeper.minutes)
        self.assertTrue(Sleeper.hour)
        self.assertTrue(Sleeper.hours)
        self.assertTrue(Sleeper.day)
        self.assertTrue(Sleeper.daily)
        self.assertTrue(Sleeper.time_range)

        self.assertRaises(TypeError, Sleeper.seconds)
        self.assertRaises(TypeError, Sleeper.minute)
        self.assertRaises(TypeError, Sleeper.within_a_minute)
        self.assertRaises(TypeError, Sleeper.minutes)
        self.assertRaises(TypeError, Sleeper.hour)
        self.assertRaises(TypeError, Sleeper.hours)
        self.assertRaises(TypeError, Sleeper.day)
        self.assertRaises(TypeError, Sleeper.daily)
        self.assertRaises(TypeError, Sleeper.time_range)
        self.assertRaises(TypeError, Sleeper.seconds, '1')

        self.assertIsNone(Sleeper.seconds(SleeperTest, 0))
        self.assertIsNone(Sleeper.minute(SleeperTest, 0))
        self.assertIsNone(Sleeper.within_a_minute(SleeperTest, 0))
        self.assertIsNone(Sleeper.minutes(SleeperTest, 0))
        self.assertIsNone(Sleeper.hour(SleeperTest, 0))
        self.assertIsNone(Sleeper.hours(SleeperTest, 0))
        self.assertIsNone(Sleeper.day(SleeperTest, 0))
        self.assertIsNone(Sleeper.daily(SleeperTest, 0))
        self.assertIsNone(Sleeper.time_range(SleeperTest, 0))


if __name__ == '__main__':
    unittest.main()



import json

from minio.datatypes import Bucket as MinioBucket


class Bucket(MinioBucket):
    def __init__(self, bucket: MinioBucket):
        self.__dict__.update(bucket.__dict__)

    def to_json(self):
        return json.dumps({k: f'{v}' for k, v in self.__dict__.items()})



import unittest

from automon.integrations.selenium.browser import SeleniumBrowser
from automon.integrations.selenium.config import SeleniumConfig


class SeleniumTest(unittest.TestCase):
    def test_SeleniumConfig(self):
        self.assertTrue(SeleniumConfig())

    def test_SeleniumBrowser(self):
        b = SeleniumBrowser()

        if b.connected:
            self.assertTrue(b)
            self.assertFalse(b.get('http://555.555.555.555'))
            self.assertTrue(b.get('http://google.com'))
            self.assertTrue(b.get_screenshot_as_png())
            self.assertTrue(b.get_screenshot_as_base64())
            self.assertTrue(b.save_screenshot())
            self.assertTrue(b.save_screenshot(folder='./'))


if __name__ == '__main__':
    unittest.main()



import os
import sys

from bs4 import BeautifulSoup

from automon import Logging
from automon.helpers import Run
from automon.helpers import Dates

from .ssid import Ssid
from .scan import ScanXml

flags = {
    '-s': 'scan for wireless networks',
    '-c': '--channel=[<arg>]    Set arbitrary channel on the card',
    '-I': '--getinfo            Print current wireless status, e.g. signal info, BSSID, port type etc.',
    '-z': '--disassociate       Disassociate from any network',
    '-s': '--scan=[<arg>]       Perform a wireless broadcast scan. Will perform a directed scan if the optional <arg> is provided',
    '-x': '--xml                Print info as XML',
    '-P': '--psk                Create PSK from specified pass phrase and SSID.',

}

log = Logging(name='Airport', level=Logging.DEBUG)


class Airport:
    def __init__(self):
        """Apple Airport Wi-Fi Utility

        A long hidden airport command line utility buried deep in Mac OS X can be used to scan for and find
        available wireless networks. This powerful tool is very helpful for network admins and systems
        administrators, but it’s handy for the average user to help discover nearby wi-fi routers as well.

        Tested on:
          - Apple M1


        # makbuk: eric$ /System/Library/PrivateFrameworks/Apple80211.framework/Versions/Current/Resources/airport -h
        # Supported arguments:
        #  -c[<arg>] --channel=[<arg>]    Set arbitrary channel on the card
        #  -z        --disassociate       Disassociate from any network
        #  -I        --getinfo            Print current wireless status, e.g. signal info, BSSID, port type etc.
        #  -s[<arg>] --scan=[<arg>]       Perform a wireless broadcast scan.
        # 				   Will perform a directed scan if the optional <arg> is provided
        #  -x        --xml                Print info as XML
        #  -P        --psk                Create PSK from specified pass phrase and SSID.
        # 				   The following additional arguments must be specified with this command:
        #                                   --password=<arg>  Specify a WPA password
        #                                   --ssid=<arg>      Specify SSID when creating a PSK
        #  -h        --help               Show this help
         """
        self._airport = '/System/Library/PrivateFrameworks/Apple80211.framework/Versions/Current/Resources/airport'

        self._connected = None
        self._channel = None
        self._network = None
        self._info = None

        self._runner = Run()

        self.scan_cmd = None
        self.scan_date = None
        self._scan_error = None
        self._scan_output = None

        self.scan_result = ScanXml()

    def __repr__(self):
        return ''

    @staticmethod
    def _command(command: str) -> list:
        return f'{command}'.split(' ')

    def create_psk(self, ssid: str, passphrase: str):
        """Create PSK from specified pass phrase and SSID."""
        if self.run(args=f'-P --ssid={ssid} --password={passphrase}'):
            return f'{self._scan_output}'.strip()
        return False

    def disassociate(self):
        """Disassociate from any network."""
        return self.run(args='-z')

    def getinfo(self):
        """Print current wireless status, e.g. signal info, BSSID, port type etc."""
        return self.run(args='-I')

    def isReady(self):
        if sys.platform == 'darwin':
            if os.path.exists(self._airport):
                return True
            else:
                log.warn(f'Airport not found! {self._airport}')
        return False

    def run(self, args: str = None):
        """Run airport"""
        if not self.isReady():
            return False

        command = f'{self._airport}'

        if args:
            command = f'{self._airport} {args}'

        self.scan_cmd = command
        self.scan_date = Dates.iso()

        try:
            log.info(command)
            if self._runner.Popen(command=command, text=True):
                self._scan_output = self._runner.stdout
                return True
        except Exception as e:
            log.error(e, raise_exception=True)

        return False

    def set_channel(self, channel: int):
        """Set arbitrary channel on the card."""
        return self.run(args=f'-c {channel}')

    def scan(self, channel: int = None, args: str = None, ssid: str = None):
        """Perform a wireless broadcast scan."""

        cmd = f'-s'

        if channel is not None:
            cmd = f'{cmd} -c {channel}'

        if ssid is not None:
            cmd = f'{cmd} {ssid}'

        if args:
            cmd = f'{cmd} {args}'

        if self.run(args=cmd):
            self._scan_output = self._runner.stdout
            self._scan_error = self._runner.stderr
            return True

        return False

    def scan_channel(self, channel: int = None):
        return self.scan(channel=channel)

    def scan_summary(self, channel: int = None, args: str = None, output: bool = True):
        if self.scan(channel=channel, args=args):
            if output:
                log.info(f'{self._scan_output}')
            return True
        return False

    @property
    def ssids(self):
        return self.scan_result.ssids

    def scan_xml(self, ssid: str = None, channel: int = None) -> [Ssid]:
        """Run scan and process xml output."""

        self.scan(ssid=ssid, args='-x', channel=channel)

        data = self._scan_output
        data = [x.strip() for x in data.splitlines()]
        data = ''.join(data)

        try:
            # root = xmltodict.parse(data)
            # root = pd.read_xml(data)
            # root = ET.fromstring(data)
            # root = etree.fromstring(data.encode())
            root = BeautifulSoup(data, "xml")

            self.scan_result.load_xml(root)

            if self.scan_result.ssids:
                return True

        except Exception as e:
            log.error(f'Scan not parsed: {e}, {self.scan_cmd}', enable_traceback=False)

        return False



import os
import importlib


class PhantomCompatableImporter:
    def __init__(self, file_name):
        self.use_case = file_name
        git_root = os.path.split(os.path.split(os.getcwd())[0])[0]
        import_file = os.path.join(git_root, self.use_case)

        temp = open(import_file, 'r').read().splitlines()
        self.compatible_import = ''.join(
            ['' if x == 'import phantom.rules as phantom' else f'{x}\n' for x in temp])

        # global spec
        # global helper
        spec = importlib.util.spec_from_loader('module', loader=None, origin=self.compatible_import)
        module = importlib.util.module_from_spec(spec)

        self.module = module

        exec(self.compatible_import, module.__dict__)


def import_playbook(playbook: str):
    playbook = playbook.strip()

    if 'master/' in playbook:
        playbook = playbook.replace('master/', '')
    return importlib.import_module(playbook)


def container():
    return {
        'id': 0
    }


def results():
    return [
        'cef_dictionary',
        'artifact_id'
    ]


def artifact(id_value: int = 0, source_data_identifier_value: str = '', results_item_1: list = results()):
    return {
            'container_id': id_value,
            'name': "User Validation",
            'contains': "{}",
            'source_data_identifier': source_data_identifier_value,
            'label': "enrich",
            'cef_value': "",
            'dedup_is_success': True,
            'run_automation': False,
            'cef_name': "",
            'cef_dictionary': results_item_1[0],
            # context (artifact id) is added to associate results with the artifact
            'context': {'artifact_id': results_item_1[1]},
        }


import os
import urllib3

from automon.log import Logging
from automon.helpers import environ

log = Logging(name='MinioConfig', level=Logging.ERROR)


class MinioConfig(object):

    def __init__(self, endpoint: str = None,
                 access_key: str = None,
                 secret_key: str = None,
                 session_token: str = None,
                 secure: bool = None,
                 region: str = None,
                 http_client: urllib3.PoolManager = None):
        """Minio config"""

        self.endpoint = endpoint or environ('MINIO_ENDPOINT', 'localhost')
        self.access_key = access_key or environ('MINIO_ACCESS_KEY')
        self.secret_key = secret_key or environ('MINIO_SECRET_KEY')
        self.session_token = session_token or environ('MINIO_SESSION_TOKEN')
        self.secure = secure or environ('MINIO_SECURE', True)
        self.region = region or environ('MINIO_REGION')
        self.http_client = http_client or environ('MINIO_HTTP_CLIENT')

        if not self.endpoint:
            log.warn(f'missing MINIO_ENDPOINT')

        if not self.access_key:
            log.warn(f'missing MINIO_ACCESS_KEY')

        if not self.secret_key:
            log.warn(f'missing MINIO_SECRET_KEY')

    def isReady(self):
        if self.endpoint and self.access_key and self.secret_key:
            return True
        return False

    def __repr__(self):
        return f'{self.__dict__}'



import neo4j
import asyncio

from neo4j import GraphDatabase
from queue import Queue

from automon.log import Logging
from automon.log.logger import logging
from automon.integrations.neo4j.cypher import Cypher

from .config import Neo4jConfig
from .results import Results

logging.getLogger('neo4j').setLevel(logging.ERROR)
log = Logging(__name__, Logging.DEBUG)


class Neo4jAsyncClient:

    def __init__(self,
                 user: str = None,
                 password: str = None,
                 hosts: list or str = None,
                 config: Neo4jConfig = None) -> neo4j:
        """Neo4j async client

        :param config: Neo4jConfig
        :param user: str
        :param password: str
        :param hosts: list
        :param encrypted: bool
        """
        self._log = Logging(Neo4jAsyncClient.__name__, Logging.DEBUG)

        self.config = config or Neo4jConfig(
            user=user,
            password=password,
            hosts=hosts
        )

        self.client = self._client()
        self.driver = self.client
        self.session = self.client.session()
        self.connected = self.isConnected()
        self.cypher = Queue()

    def __repr__(self):
        return f'{self.__dict__}'

    def _check_connection(self):
        if self.client:
            return True
        return False

    def _client(self):
        try:
            client = GraphDatabase.driver(
                uri=self.config.NEO4J_HOST,
                auth=(self.config.NEO4J_USER, self.config.NEO4J_PASSWORD))
            log.info(f'Connected to neo4j server: {self.config.NEO4J_HOST}')
            return client

        except Exception as e:
            self._log.error(f'Cannot connect to neo4j server: {self.config.NEO4J_HOST}, {e}',
                            enable_traceback=False, raise_exception=True)

        return False

    def _consumer(self):
        return

    def _producer(self):
        return

    def isConnected(self):
        return self._check_connection()

    def run(self):
        """Send cyphers in queue"""
        try:
            while not self.cypher.empty():
                cypher = self.cypher.get_nowait()
                self._log.debug(f'cypher: {cypher}')
                self.session.run(cypher)
            return True
        except Exception as e:
            self._log.error(f'{e}')

        return False



import unittest

from automon.integrations.elasticsearch.snapshots import Snapshot, SnapshotError, ElasticsearchSnapshotMonitor


class SnapshotTest(unittest.TestCase):

    def test_Snapshot(self):
        self.assertTrue(Snapshot)
        self.assertTrue(Snapshot({}))
        self.assertEqual(Snapshot({}), Snapshot({}))
        self.assertNotEqual(Snapshot({}), None)

    def test_SnapshotError(self):
        error = {
            'error': {
                'test': 'test',
                'caused_by': {
                    'type': None,
                    'reason': None,
                    'caused_by': {
                        'type': None,
                        'reason': None
                    }
                }
            }
        }

        self.assertTrue(SnapshotError)
        self.assertTrue(SnapshotError({}))
        self.assertEqual(SnapshotError({}), SnapshotError({}))
        self.assertNotEqual(SnapshotError({}), None)
        self.assertTrue(SnapshotError(error))

    def test_ElasticsearchSnapshotMonitor(self):
        e = ElasticsearchSnapshotMonitor(elasticsearch_repository='found-snapshots')

        if e.connected:
            self.assertTrue(ElasticsearchSnapshotMonitor)
            self.assertTrue(e)
            self.assertTrue(e.check_snapshots())
        else:
            self.assertTrue(ElasticsearchSnapshotMonitor)
            self.assertTrue(e)
            self.assertFalse(e.check_snapshots())


if __name__ == '__main__':
    unittest.main()



import os

import tempfile

from urllib.parse import urlparse

from automon.log import Logging
from automon.helpers.dates import Dates
from automon.helpers.sleeper import Sleeper
from automon.helpers.sanitation import Sanitation
from automon.integrations.selenium.config import SeleniumConfig


class SeleniumBrowser:

    def __init__(self, config: SeleniumConfig = None):
        self._log = Logging(name=SeleniumBrowser.__name__, level=Logging.DEBUG)

        self.config = config or SeleniumConfig()
        self.webdriver = self.config.webdriver
        self.connected = False

        try:
            self.browser = self.webdriver.Chrome()
            self.connected = True
        except Exception as e:
            self._log.error(f'Unable to spawn browser: {e}')

    def get(self, url: str):
        try:
            self.browser.get(url)
            return True
        except Exception as e:
            self._log.error(f'Error getting {url}: {e}')

        return False

    def get_screenshot_as_png(self):
        return self.browser.get_screenshot_as_png()

    def get_screenshot_as_base64(self):
        return self.browser.get_screenshot_as_base64()

    def save_screenshot(self, filename: str = None, prefix: str = None, folder: str = None):

        if not filename:
            filename = self._screenshot_name(prefix)

        if not folder:
            path = os.path.abspath(tempfile.gettempdir())
        else:
            path = os.path.abspath(folder)

        if not os.path.exists(path):
            os.makedirs(path)

        save = os.path.join(path, filename)

        self._log.info(f'Saving screenshot to: {save}')

        return self.browser.save_screenshot(save)

    def _screenshot_name(self, prefix=None):
        """Generate a unique filename

        :param browser:
        :param prefix: prefix filename with a string
        :return:
        """
        title = self.browser.title
        url = self.browser.current_url
        hostname = urlparse(url).hostname

        hostname_ = Sanitation.ascii_numeric_only(hostname)
        title_ = Sanitation.ascii_numeric_only(title)
        timestamp = Dates.filename_timestamp()

        if prefix:
            prefix = Sanitation.safe_string(prefix)
            return f'{prefix}_{hostname_}_{title_}_{timestamp}.png'

        return f'{hostname_}_{title_}_{timestamp}.png'

    def new_resolution(self, width=1920, height=1080, device_type='web'):

        if device_type == 'pixel3':
            width = 1080
            height = 2160

        if device_type == 'web-small' or device_type == '800x600':
            width = 800
            height = 600

        if device_type == 'web-small-2' or device_type == '1024x768':
            width = 1024
            height = 768

        if device_type == 'web-small-3' or device_type == '1280x960':
            width = 1280
            height = 960

        if device_type == 'web-small-4' or device_type == '1280x1024':
            width = 1280
            height = 1024

        if device_type == 'web' or device_type == '1920x1080':
            width = 1920
            height = 1080

        if device_type == 'web-2' or device_type == '1600x1200':
            width = 1600
            height = 1200

        if device_type == 'web-3' or device_type == '1920x1200':
            width = 1920
            height = 1200

        if device_type == 'web-large' or device_type == '2560x1400':
            width = 2560
            height = 1400

        if device_type == 'web-long' or device_type == '1920x3080':
            width = 1920
            height = 3080

        self.browser.set_window_size(width, height)

    def close(self):
        self.browser.close()

    def quit(self):
        self.browser.close()
        self.browser.quit()
        self.browser.stop_client()


if __name__ == "__main__":
    pass



from os import getenv

from automon import Logging

log = Logging(name='GmailConfig', level=Logging.DEBUG)


class GmailConfig:
    def __init__(self, endpoint: str = None,
                 api_key: str = None,
                 user: str = None,
                 password: str = None):
        """Gmail config"""
        self.endpoint = endpoint or getenv('GMAIL_ENDPOINT') or 'https://gmail.googleapis.com'
        self.api_key = api_key or getenv('GMAIL_API_KEY')
        self.userId = user or getenv('GMAIL_USER')
        self.password = password or getenv('GMAIL_PASSWORD')

    def __repr__(self):
        return f'{self.__dict__}'



import bcrypt
import random

from flask import (redirect, url_for, session, flash)
from flask import (session, redirect, url_for)
from flask_login import (logout_user, login_user, login_fresh,
                         current_user, LoginManager, UserMixin)

from .auth_creds import credential_db


def login_manager_wrapper(app):
    """ Wrapper for flask_login.LoginManager()

    :param app: Flask app name to initialize
    :return: flask_login.LoginManager object
    """

    login_manager = LoginManager()
    login_manager.init_app(app)
    login_manager.session_protection = "strong"

    # login_manager = app_userloader.login_init()
    # login_manager.login_view = 'login'

    return login_manager


def load_user(user_id):
    """ Callback to reload the user object

    :param user_id: User.id to look up
    :return: user UserMixin class
    """

    # user = User()
    user = UserMixin()
    user.id = user_id
    return user


def login(request):
    """ This authenticates the user login with the internal user database

    :return: True if authenticated; False, with error message
    """

    if request.method == 'POST':

        if 'username' in request.form:
            username = str(request.form['username']).strip().lower()

        if 'password' in request.form:
            password = request.form['password'].encode()

        if 'remember' in request.form:
            remember = True
        else:
            remember = False

        if 'force' in request.form:
            # Disable force flag
            # user_id will need to be active to proceed
            # force = True
            force = False
        else:
            force = False

        credentials = credential_db()

        errors = [
            '''Well, that didn't work''',
            '''Everything is wrong''',
            '''Did not work''',
            '''Hello?''',
            '''Why don't you try again?''',
            '''Try again''',
            '''Login: ixzd2@skynt Password: lost''',
            '''Password is password''',
            '''I'm sorry Dave, but I can't do that''',
            '''This is borderline harrassment''',
            '''Not cool''',
            '''If you're looking for something, you're doing it wrong''',
            '''Aren't you a persistent one?''',
            '''I'm sorry, what are you looking for?''',
            '''Find a hero in you''',
            '''This isn't getting anywhere''',
            '''Let's play a game''',
            ''':('''
        ]

        if username == 'ixzd2@skynt' and \
                bcrypt.checkpw(
                    password,
                    b'$2b$12$j.LDt.8CQ7BMiRAgYWEsSunXAbeiOo9qNJvqvZJ3fbJ1MX7yxu4Zu'):
            error = 'haha just kidding'

            return False, error

        elif username not in credentials.keys() or bcrypt.checkpw(password, credentials[username]) is False:
            error = random.choice(errors)

            return current_user.is_authenticated, error

        elif bcrypt.checkpw(password, credentials[username]):

            # Authenticate session
            user = UserMixin()
            user.id = username  # id == user_id

            login_user(user, remember=remember, force=force)

            # flash('Logged in successfully')

            return current_user.is_authenticated, None

    return current_user.is_authenticated, None


def fresh():
    """ Returns True if the user login is fresh

    Is the current login fresh?
    """
    return login_fresh()


def logout():
    """ This logs a user out via flask_login.logout_user function
    """
    return logout_user()


def logged_in():
    """ If the user is logged in return True, otherwise return False

    True if logged in; False if not logged in
    """

    if current_user.is_authenticated:
        return True

    return False


def current_user_info():
    """ Return the current_user session

    Current user.id from Flask.session is returned
    """

    return current_user


def request_headers(request):
    """ Return request headers
    """

    try:
        real_ip = request['X-Real-IP']
        headers = request
        headers['Host'] = real_ip
        return dict(request.headers)
    except:
        return dict(request.headers)


def request_environ(request):
    """ Return request headers
    """

    return dict(request.environ)



import os
import logging

from automon.log import Logging
from automon.helpers.sanitation import Sanitation as S

logging.getLogger('elasticsearch').setLevel(logging.ERROR)
logging.getLogger('urllib3.connectionpool').setLevel(logging.ERROR)


class ElasticsearchConfig:

    def __init__(self, host: str = None,
                 cloud_id: str = None,
                 user: str = '',
                 password: str = '',
                 api_key: tuple = None,
                 api_key_id: str = None,
                 api_key_secret: str = None,
                 request_timeout: int = 1,
                 http_auth: tuple = None,
                 use_ssl: bool = True,
                 verify_certs: bool = True,
                 connection_class: object = None,
                 proxy=None):
        """elasticsearch config"""

        self._log = Logging(ElasticsearchConfig.__name__, Logging.DEBUG)

        # hosts
        self.ELASTICSEARCH_HOST = host or os.getenv('ELASTICSEARCH_HOSTS')
        self.ELASTICSEARCH_CLOUD_ID = cloud_id or os.getenv('ELASTICSEARCH_CLOUD_ID')

        # auth
        self.ELASTICSEARCH_USER = user or os.getenv('ELASTICSEARCH_USER')
        self.ELASTICSEARCH_PASSWORD = password or os.getenv('ELASTICSEARCH_PASSWORD')
        self.ELASTICSEARCH_API_KEY = api_key or os.getenv('ELASTICSEARCH_API_KEY')
        self.ELASTICSEARCH_API_KEY_ID = api_key_id or os.getenv('ELASTICSEARCH_API_KEY_ID')
        self.ELASTICSEARCH_API_KEY_SECRET = api_key_secret or os.getenv('ELASTICSEARCH_API_KEY_SECRET')

        if self.ELASTICSEARCH_API_KEY_ID and self.ELASTICSEARCH_API_KEY_SECRET \
                and not self.ELASTICSEARCH_API_KEY:
            self.ELASTICSEARCH_API_KEY = (self.ELASTICSEARCH_API_KEY_ID, self.ELASTICSEARCH_API_KEY_SECRET)

        # options
        self.ELASTICSEARCH_PROXY = proxy or os.getenv('ELASTICSEARCH_PROXY')
        self.ELASTICSEARCH_REQUEST_TIMEOUT = request_timeout or os.getenv('ELASTICSEARCH_REQUEST_TIMEOUT')
        self.request_timeout = self.ELASTICSEARCH_REQUEST_TIMEOUT
        self.use_ssl = use_ssl
        self.verify_certs = verify_certs
        self.connection_class = connection_class

        if self.ELASTICSEARCH_USER and self.ELASTICSEARCH_PASSWORD:
            self.http_auth = (self.ELASTICSEARCH_USER, self.ELASTICSEARCH_PASSWORD)
        else:
            self.http_auth = http_auth

    def __repr__(self):
        return f'{self.__dict__}'

    def __eq__(self, other):
        if not isinstance(other, ElasticsearchConfig):
            self._log.warn(f'Not implemented')
            return NotImplemented

        return self.ELASTICSEARCH_HOST == other.ELASTICSEARCH_HOST


class SnapshotBot:
    def __init__(self):
        self.slack_name = 'Elasticsearch Daily Monitor bot'
        self.es_hosts = ElasticsearchConfig().ELASTICSEARCH_HOST
        self.es_repository = os.getenv('ELASTICSEARCH_REPOSITORY')
        self.snapshots_prefix = os.getenv('SNAPSHOTS_PREFIX')


class JVMBot:
    def __init__(self):
        self.slack_name = 'Elasticsearch JVM Monitor bot'
        self.es_hosts = ElasticsearchConfig().ELASTICSEARCH_HOST



from automon.integrations.requests import RequestsClient

from .config import GmailConfig


class GmailClient:

    def __init__(self, api_key: str = None, user: str = None, password: str = None, config: GmailConfig = None):
        self.config = config or GmailConfig(user=user, password=password, api_key=api_key)
        self.endpoint = self.config.endpoint
        self.userId = self.config.userId

        self.client = RequestsClient()

    def _base_url(self, url: str):
        return f'{self.endpoint}{url}'

    def draft_create(self):
        """Creates a new draft with the DRAFT label."""
        api = [
            f'/gmail/v1/users/{self.userId}/drafts',
            f'/upload/gmail/v1/users/{self.userId}/drafts'
        ]
        return self.client.post(self._base_url(api[0]))

    def draft_delete(self, id: int):
        """Immediately and permanently deletes the specified draft."""
        api = f'/gmail/v1/users/{self.userId}/drafts/{id}'
        return self.client.delete(self._base_url(api))

    def draft_get(self, id: int):
        """Gets the specified draft."""
        api = f'/gmail/v1/users/{self.userId}/drafts/{id}'
        return self.client.get(self._base_url(api))

    def draft_list(self):
        """Lists the drafts in the user's mailbox."""
        api = f'/gmail/v1/users/{self.userId}/drafts'
        return self.client.get(self._base_url(api))

    def draft_send(self):
        """Sends the specified, existing draft to the recipients in the To, Cc, and Bcc headers."""
        api = [
            f'/gmail/v1/users/{self.userId}/drafts/send',
            f'/upload/gmail/v1/users/{self.userId}/drafts/send'
        ]
        return self.client.post(self._base_url(api[0]))

    def draft_update(self, id: int):
        api = [
            f'/gmail/v1/users/{self.userId}/drafts/{id}'
            f'/upload/gmail/v1/users/{self.userId}/drafts/{id}'
        ]
        return self.client.put(self._base_url(api[0]))

    def history_list(self):
        """Lists the history of all changes to the given mailbox."""
        api = f'/gmail/v1/users/{self.userId}/history'
        return self.client.get(self._base_url(api))

    def labels_create(self, label: str):
        """Creates a new label."""
        api = f'/gmail/v1/users/{self.userId}/labels'
        return self.client.post(self._base_url(api))

    def labels_delete(self, id: int):
        """Immediately and permanently deletes the specified label and
        removes it from any messages and threads that it is applied to."""
        api = f'/gmail/v1/users/{self.userId}/labels/{id}'
        return self.client.delete(self._base_url(api))

    def labels_get(self, id: int):
        """Gets the specified label."""
        api = f'/gmail/v1/users/{self.userId}/labels/{id}'
        return self.client.get(self._base_url(api))

    def labels_list(self):
        """Lists all labels in the user's mailbox."""
        api = f'/gmail/v1/users/{self.userId}/labels'
        return self.client.get(self._base_url(api))

    def labels_patch(self, id: int):
        """Patch the specified label."""
        api = f'/gmail/v1/users/{self.userId}/labels/{id}'
        return self.client.patch(self._base_url(api))

    def labels_update(self, id: int):
        """Updates the specified label."""
        api = f'/gmail/v1/users/{self.userId}/labels/{id}'
        return self.client.put(self._base_url(api))

    def users_watch(self):
        """Set up or update a push notification watch on the given user mailbox."""
        api = f'/gmail/v1/users/{self.userId}/watch'
        return self.client.post(self._base_url(api))

    def users_getProfile(self):
        """Gets the current user's Gmail profile."""
        api = f'/gmail/v1/users/{self.userId}/profile'
        return self.client.get(self._base_url(api))

    def users_stop(self):
        """Stop receiving push notifications for the given user mailbox."""
        api = f'/gmail/v1/users/{self.userId}/stop'
        return self.client.post(self._base_url(api))



import re

from urllib.parse import urlencode
from datetime import datetime, timezone

from automon.log import Logging

log = Logging(name='Cypher', level=Logging.DEBUG)


class Node:

    def __init__(self,
                 node: str = None,
                 label: str = None,
                 prop: str = None,
                 value: str = None):
        """Basic Node

        ( node :`label` { `prop`: "value" } )
        """
        self.node = node
        self.label = Cypher.assert_label(label)
        self.prop = Cypher.assert_property(prop)
        self.value = value

        if self.prop and self.value:
            prop_val = f'{self.prop}: "{self.value}"'
        else:
            prop_val = ''

        self.cypher = f'( {self.node} {self.label} {{ {prop_val} }} )'

    def __repr__(self):
        return self.cypher


class Cypher:

    def __init__(self):
        self.cypher = None
        self.cypher_list = []

    def __repr__(self):
        return f'{self.cypher}'

    def add_property(self, prop: str, value: str, node: str = None) -> str:
        """add property

        return: Node.prop = "value"
        """
        prop = self.assert_property(prop)
        cypher = f'{node}.{prop} = "{value}"'
        return cypher

    @staticmethod
    def assert_label(label: str) -> str:
        """Make sure neo4j label is formatted correctly"""

        if not label:
            return ''

        if re.search('[:]', label):
            log.warn(f"Invalid label '{label}': Colon is not needed here")
            label = label.replace(':', '')

        if re.search('[`]', label):
            label = label.replace('`', '')

        if not re.search('[a-zA-Z]', label[0]):  # First letter of a label must be a letter
            log.error(f"Invalid label '{label}': First character of Neo4j :LABEL must be a letter")
        else:
            return f':`{label}`'  # :`LABEL`

    @staticmethod
    def assert_property(prop) -> str:
        prop = f'{prop}'
        prop = prop.strip()

        i = 0
        for char in prop:
            if char == '`':
                i += 1

        if i == 2:
            return prop

        if i < 2:
            prop = prop.replace('`', '')
            prop = f'`{prop}`'

        return prop

    def build_cypher(self, cypher: str):
        """Build cypher line by line"""
        self.cypher += f'{cypher} '
        return self.cypher

    def consolidate(self) -> str:
        """Join cypher queries list into a string"""
        return ' '.join(self.cypher_list).strip()

    def create(self, prop: str, value: str, node: str = None, label: str = ''):
        """Create a node

        CREATE ( node :`label` { `prop`: "value" } )
        """
        n = Node(prop=prop, value=value, node=node, label=label)
        cypher = f'CREATE {n} \n'
        return cypher

    def create_dict(self, prop: str, value: str, data: dict, node: str = None, label: str = ''):
        """Create a node from dict

        CREATE (Node :`human` { `name`: "finn" })
        SET
        Node.first_seen = "2021-02-25T03:19:47.438596+00:00",
        Node.first_seen_ts = timestamp()
        SET
        Node.prop = "value",
        Node.prop = "value"
        SET
        Node.last_seen = "2021-02-25T03:19:47.438901+00:00",
        Node.last_seen_ts = timestamp()
        SET
        Node.prop = "value"
        RETURN *
        """
        cypher = self.create(node=node, label=label, prop=prop, value=value)
        cypher += self.set()
        cypher += self.timestamp_first_seen()

        cypher += self.set()
        cypher += self.dict_to_property(data=data, node=node)

        cypher += self.set()
        cypher += self.timestamp_last_seen(node=node)

        cypher += self.set()
        cypher += self.dict_to_property(data=data, node=node)
        cypher += self.return_all()
        return cypher

    def cypher_end(self):
        """End of cypher"""
        return ') \n'

    def delete_all(self):
        """Delete all nodes and relationships"""
        return 'MATCH (n) DETACH DELETE n'

    def delete_node(self, prop: str, value: str, node: str = None) -> str:
        """Delete all matching nodes and its relationships"""
        # MATCH (n {name: 'Andy'})
        # DETACH DELETE n
        cypher = self.match(prop=prop, value=value, node=node)
        cypher += f'DETACH DELETE {node}'
        return cypher

    def dict_to_property(self, data: dict, node: str = None) -> str:
        """Dict to cypher

        return: node.`key` = "value"
        """

        if not data:
            return f'{{}} \n'

        data = self.prepare_dict(data)
        cypher = []
        i = 1
        for prop in data.keys():
            value = data.get(prop)
            prop = self.assert_property(prop)
            query = self.add_property(node=node, prop=prop, value=value)

            if i < len(data.keys()):
                cypher.append(query + ',\n')
            else:
                cypher.append(query)
            i += 1

        cypher = ''.join(cypher)
        cypher = f'{cypher} \n'
        return cypher

    def match(self, prop: str, value: str, node: str = None, label: str = ''):
        """Match a node

        MATCH (node :label {`prop`: "value"})
        """
        prop = self.assert_property(prop)
        label = self.assert_label(label)

        return f'MATCH ({node} {label} {{ {prop}: "{value}" }}) \n'

    def merge(self, prop: str = None, value: str = None, node: str = None, label: str = ''):
        """Merge a node

        MERGE ( node :`label` { `prop`: "value" } )
        """
        n = Node(prop=prop, value=value, node=node, label=label)
        cypher = f'MERGE {n} \n'
        return cypher

    def merge_dict(self, prop: str, value: str, data: dict, node: str = None, label: str = '') -> str:
        """Merge a node from a dict

        MERGE (Node :`human` { `name`: "finn" })
        ON CREATE
        SET
        Node.first_seen = "2021-02-25T03:19:47.438596+00:00",
        Node.first_seen_ts = timestamp()
        ON CREATE
        SET
        Node.prop = "value",
        Node.prop = "value"
        ON MATCH
        SET
        Node.updated = "2022-01-24T03:41:31.160885+00:00",
        Node.updated_ts = timestamp()
        ON MATCH
        SET
        Node.prop = "value",
        Node.prop = "value"
        RETURN *
        """
        cypher = self.merge(prop=prop, value=value, node=node, label=label)
        cypher += self.on_create()
        cypher += self.timestamp_first_seen()

        cypher += self.on_create()
        cypher += self.dict_to_property(data=data, node=node)

        cypher += self.on_match()
        cypher += self.timestamp_updated()

        cypher += self.on_match()
        cypher += self.dict_to_property(data=data, node=node)
        cypher += self.return_all()
        return cypher

    def on_match(self):
        cypher = 'ON MATCH \n'
        cypher += self.set()
        return cypher

    def on_create(self):
        cypher = 'ON CREATE \n'
        cypher += self.set()
        return cypher

    @staticmethod
    def prepare_dict(blob: dict) -> dict:
        """All inputs first needs to be dicts"""
        try:
            return dict(blob)
        except Exception as _:
            return dict(raw=urlencode(blob))

    def return_all(self):
        """RETURN *"""
        return 'RETURN *'

    def return_node(self, node: str = None):
        """RETURN node"""
        return f'RETURN {node}'

    def relationship(self,
                     A_node: str = 'A', A_label: str = None, A_prop: str = None, A_value: str = None,
                     B_node: str = 'B', B_label: str = None, B_prop: str = None, B_value: str = None,
                     WHERE: str = None, label: str = None, node: str = 'r', direction: str = '->'):
        """Create relationship between two existing nodes

        MATCH (A_node :A_label {`A_prop`: "A_value"})
        MATCH (B_node :B_label {`B_prop`: "B_value"})

        MERGE (a)-[r :label]->(b)
        RETURN *
        """

        A_label = self.assert_label(A_label)
        B_label = self.assert_label(B_label)
        label = self.assert_label(label)

        cypher_a = self.match(node=A_node, label=A_label, prop=A_prop, value=A_value)
        cypher_b = self.match(node=B_node, label=B_label, prop=B_prop, value=B_value)
        cypher_where = f'WHERE {WHERE} \n'
        begin, end = self._relationship_direction(direction)

        cypher = cypher_a
        cypher += cypher_b
        if cypher_where:
            cypher += cypher_where
        cypher += f'MERGE ({A_node}) {begin} [{node} {label}] {end} ({B_node}) \n'
        cypher += self.return_all()

        return cypher

    def _relationship_direction(self, direction: str = '->'):
        """Sets direction of node relationship

        -> or > or >>
        <- or < or <<
        <-> or <>
        """

        begin = '-'
        end = '->'

        if direction == '->' or direction == '>' or direction == '>>':
            begin = '-'
            end = '->'

        if direction == '<-' or direction == '<' or direction == '<<':
            begin = '<-'
            end = '-'

        if direction == '<->' or direction == '<>' or direction == '<<>>':
            begin = '<-'
            end = '->'

        return begin, end

    def unique(self, node: str = None, label: str = ''):
        label = self.assert_label(label)
        return f'UNIQUE ({node} {label} {{'

    def updated(self, node: str = None):
        """Node last_seen property"""
        node = node
        time = datetime.now(tz=timezone.utc).isoformat()
        self.cypher_list.append(f'SET {node}.last_seen = "{time}"')
        self.cypher_list.append(f'SET {node}.last_seen_ts = timestamp()')

    @staticmethod
    def set():
        return 'SET \n'

    def timestamp_first_seen(self, node: str = None) -> str:
        """Node first_seen property

        Node.first_seen = "2022-01-24T03:41:31.160885+00:00",
        Node.first_seen_ts = timestamp()
        """
        time = datetime.now(tz=timezone.utc).isoformat()

        cypher = f'{node}.first_seen = "{time}", \n'
        cypher += f'{node}.first_seen_ts = timestamp() \n'
        return cypher

    def timestamp_last_seen(self, node: str = None) -> str:
        """Node last_seen property

        Node.last_seen = "2022-01-24T03:41:31.160885+00:00",
        Node.last_seen_ts = timestamp()
        """
        time = datetime.now(tz=timezone.utc).isoformat()
        cypher = f'{node}.last_seen = "{time}", \n'
        cypher += f'{node}.last_seen_ts = timestamp() \n'
        return cypher

    def timestamp_updated(self, node: str = None) -> str:
        """Node updated property

        Node.updated = "2022-01-24T03:41:31.160885+00:00",
        Node.updated_ts = timestamp()
        """
        time = datetime.now(tz=timezone.utc).isoformat()
        cypher = f'{node}.updated = "{time}", \n'
        cypher += f'{node}.updated_ts = timestamp() \n'
        return cypher



import os


def environ(env_var: str, default: any = None):
    env = os.getenv(env_var)
    if env:
        if f'{env}'.lower() == 'true':
            return True
        if f'{env}'.lower() == 'false':
            return False
        return env
    return default



from sentry_sdk import set_level

from automon.helpers.os import environ


class SentryConfig(object):
    def __init__(self, dsn: str = None,
                 environment: str = None,
                 release: str = None,
                 debug: bool = False,
                 sample_rate: float = None,
                 request_bodies: str = None,
                 level: str = None):
        self.SENTRY_DSN = dsn or environ('SENTRY_DSN')
        self.SENTRY_ENVIRONMENT = environment or environ('SENTRY_ENVIRONMENT')
        self.SENTRY_RELEASE = release or environ('SENTRY_RELEASE')

        self.level = level or 'debug'
        set_level(self.level)

        # common options
        self.dsn = self.SENTRY_DSN
        self.environment = self.SENTRY_ENVIRONMENT or 'dev'
        self.release = self.SENTRY_RELEASE
        self.debug = debug
        self.sample_rate = sample_rate or 1.0
        self.max_breadcrumbs = None
        self.attach_stacktrace = None
        self.send_default_pii = None
        self.server_name = None
        self.in_app_include = None
        self.in_app_exclude = None
        self.request_bodies = request_bodies or 'always'
        self.with_locals = None
        self.ca_certs = None

        # integration configuration
        self.integrations = None
        self.default_integrations = None

        # hooks
        self.before_send = None
        self.before_breadcrumb = None

        # transport options
        self.transport = None
        self.http_proxy = None
        self.https_proxy = None
        self.shutdown_timeout = None

        # tracing options
        self.traces_sample_rate = None
        self.traces_sampler = None

    def __repr__(self):
        return f'{self.__dict__}'

    def setLevel(self, level):
        self.level = level
        return set_level(self.level)



from .container import Container


class ResultsContainer(object):
    count: int
    num_pages: int
    data: [Container]

    def __init__(self, results: dict = {}):
        self.data = None
        self.__dict__.update(results)

        if 'data' in results.keys():
            self.data = [Container(x) for x in self.data]

    def __repr__(self):
        return f'{self.__dict__}'



from automon import Logging

from .common import Scaffolding

log = Logging('Artifact', level=Logging.CRITICAL)


class Artifact(Scaffolding):
    def __init__(self, artifact: dict):
        self.__dict__.update(artifact)



import sentry_sdk

from sentry_sdk import capture_exception as _capture_exception
from sentry_sdk import capture_event as _capture_event
from sentry_sdk import capture_message as _capture_message

from .config import SentryConfig


class SentryClient(object):
    def __init__(self, dsn: str = None, config: SentryConfig = None, *args, **kwargs):
        self.config = config or SentryConfig(dsn=dsn)

        self.sentry = sentry_sdk.init(
            dsn=self.config.dsn,
            environment=self.config.environment,
            release=self.config.release,
            debug=self.config.debug,
            sample_rate=self.config.sample_rate,
            max_breadcrumbs=self.config.max_breadcrumbs,
            attach_stacktrace=self.config.attach_stacktrace,
            send_default_pii=self.config.send_default_pii,
            server_name=self.config.server_name,
            in_app_include=self.config.in_app_include,
            in_app_exclude=self.config.in_app_exclude,
            request_bodies=self.config.request_bodies,
            with_locals=self.config.with_locals,
            ca_certs=self.config.ca_certs,
            integrations=self.config.integrations,
            default_integrations=self.config.default_integrations,
            before_send=self.config.before_send,
            before_breadcrumb=self.config.before_breadcrumb,
            transport=self.config.transport,
            http_proxy=self.config.http_proxy,
            https_proxy=self.config.https_proxy,
            shutdown_timeout=self.config.shutdown_timeout,
            traces_sample_rate=self.config.traces_sample_rate,
            traces_sampler=self.config.traces_sampler,
            *args, **kwargs
        )

    def __repr__(self):
        return f'{self.__dict__}'

    def isConnected(self):
        if self.config.dsn:
            return True
        return False

    def setLevel(self, level):
        return self.config.setLevel(level)

    def capture_exception(self, exception):
        if self.isConnected():
            return _capture_exception(exception)
        return False

    def capture_event(self, message: str, level):
        if self.isConnected():
            return _capture_event(dict(
                message=message,
                level=level
            ))
        return False

    def capture_message(self, message):
        if self.isConnected():
            return _capture_message(message)
        return False

    def error(self, msg: str):
        self.setLevel('error')
        return self.capture_message(f'{msg}')

    def warning(self, msg: str):
        self.setLevel('warning')
        return self.capture_message(f'{msg}')

    def warn(self, msg: str):
        return self.warning(msg=msg)

    def info(self, msg: str):
        self.setLevel('info')
        return self.capture_message(f'{msg}')

    def debug(self, msg: str):
        self.setLevel('debug')
        return self.capture_message(f'{msg}')

    def critical(self, msg: str):
        self.setLevel('critical')
        return self.capture_message(f'{msg}')



import json
import datetime

from automon import Logging

from .common import Scaffolding

log = Logging('Container', level=Logging.CRITICAL)


class Container(Scaffolding):
    artifact_count: int
    start_time: datetime
    id: int

    def __init__(self, container: dict):
        self.artifact_count = None
        self.start_time = None
        self.id = None
        self.__dict__.update(container)

    def __repr__(self):
        if len(self.to_json()) > 100:
            return json.dumps(dict(
                name=self.name,
                label=self.label,
                status=self.status,
                id=self.id,
                artifact_count=self.artifact_count,
                create_time=self.create_time
            ))
        return self.to_json()



import unittest

from automon.healthcheck import Healthcheck


class HealthcheckTest(unittest.TestCase):
    def test_Healthcheck(self):
        self.assertTrue(Healthcheck())
        self.assertFalse(Healthcheck().up)
        self.assertFalse(Healthcheck().check())

# if __name__ == '__main__':
#     unittest.main()



from automon.integrations.neo4j.cypher import Cypher


# from core.helpers import cryptography


def http_header(headers):
    # [print(x) for x in auth.request_headers(request)]

    # token = cryptography.hash_key(sorted([x for x in headers]))

    args = dict(
        blob=sorted(headers),
        label='Headers'
    )

    Cypher.prepare_dict(**args)



from enum import Enum

from automon import Logging

log = Logging(level=Logging.DEBUG)


class AgeRange(Enum):
    """Please use person.ageRanges instead"""
    log.warn(DeprecationWarning)

    AGE_RANGE_UNSPECIFIED = 'AGE_RANGE_UNSPECIFIED'
    LESS_THAN_EIGHTEEN = 'LESS_THAN_EIGHTEEN'
    EIGHTEEN_TO_TWENTY = 'EIGHTEEN_TO_TWENTY'
    TWENTY_ONE_OR_OLDER = 'TWENTY_ONE_OR_OLDER'


class ContentType(Enum):
    """
    :CONTENT_TYPE_UNSPECIFIED: Unspecified
    :TEXT_PLAIN: Plain text.
    :TEXT_HTML: HTML text.
    """
    CONTENT_TYPE_UNSPECIFIED = 'CONTENT_TYPE_UNSPECIFIED'
    TEXT_PLAIN = 'TEXT_PLAIN'
    TEXT_HTML = 'TEXT_HTML'


class Date(object):
    """
    :year: Year of the date. Must be from 1 to 9999, or 0 to specify a date without a year.
    :month: Month of a year. Must be from 1 to 12, or 0 to specify a year without a month and day.
    :day: Day of a month. Must be from 1 to 31 and valid for the year and month, or 0 to specify a
        year by itself or a year and month where the day isn't significant.
    """
    year: int
    month: int
    dat: int


class KeywordType(Enum):
    TYPE_UNSPECIFIED = 'TYPE_UNSPECIFIED'
    OUTLOOK_BILLING_INFORMATION = 'OUTLOOK_BILLING_INFORMATION'
    OUTLOOK_DIRECTORY_SERVER = 'OUTLOOK_DIRECTORY_SERVER'
    OUTLOOK_KEYWORD = 'OUTLOOK_KEYWORD'
    OUTLOOK_MILEAGE = 'OUTLOOK_MILEAGE'
    OUTLOOK_PRIORITY = 'OUTLOOK_PRIORITY'
    OUTLOOK_SENSITIVITY = 'OUTLOOK_SENSITIVITY'
    OUTLOOK_SUBJECT = 'OUTLOOK_SUBJECT'
    OUTLOOK_USER = 'OUTLOOK_USER'
    HOME = 'HOME'
    WORK = 'WORK'
    OTHER = 'OTHER'


class NicknameType(Enum):
    """
    :DEFAULT: Generic nickname.
    :MAIDEN_NAME: (depreciated)
    :INITIALS: (depreciated)
    :GPLUS: (depreciated)
    :OTHER_NAME: (depreciated)
    :ALTERNATE_NAME: (depreciated)
    :SHORT_NAME: (depreciated)
    """
    DEFAULT = 'DEFAULT'
    MAIDEN_NAME = 'MAIDEN_NAME'
    INITIALS = 'INITIALS'
    GPLUS = 'GPLUS'
    OTHER_NAME = 'OTHER_NAME'
    ALTERNATE_NAME = 'ALTERNATE_NAME'
    SHORT_NAME = 'SHORT_NAME'


class ObjectType(Enum):
    OBJECT_TYPE_UNSPECIFIED = 'OBJECT_TYPE_UNSPECIFIED'
    PERSON = 'PERSON'
    PAGE = 'PAGE'


class SourceType(Enum):
    SOURCE_TYPE_UNSPECIFIED = 'SOURCE_TYPE_UNSPECIFIED'
    ACCOUNT = 'ACCOUNT'
    PROFILE = 'PROFILE'
    DOMAIN_PROFILE = 'DOMAIN_PROFILE'
    CONTACT = 'CONTACT'
    OTHER_CONTACT = 'OTHER_CONTACT'
    DOMAIN_CONTACT = 'DOMAIN_CONTACT'


class UserType(Enum):
    USER_TYPE_UNKNOWN = 'USER_TYPE_UNKNOWN'
    GOOGLE_USER = 'GOOGLE_USER'
    GPLUS_USER = 'GPLUS_USER'
    GOOGLE_APPS_USER = 'GOOGLE_APPS_USER'


class ProfileMetadata(object):
    objectType: ObjectType
    userTypes: [UserType]


class Source(object):
    type: SourceType
    etag: str
    updateTime: str
    profileMetadata: ProfileMetadata


class FieldMetadata(object):
    primary: bool
    sourcePrimary: bool
    verified: bool
    source: Source


class FileMetadata(object):
    primary: bool
    sourcePrimary: bool
    verified: bool
    source: Source


class Address(object):
    metadata: FieldMetadata
    formattedValue: str
    type: str
    formattedType: str
    poBox: str
    streetAddress: str
    extendedAddress: str
    city: str
    region: str
    postalCode: str
    country: str
    countryCode: str


class AgeRangeType(object):
    metadata: FieldMetadata
    ageRange: AgeRange


class Biography(object):
    metadata: FieldMetadata
    value: str
    contentType: ContentType


class Birthday(object):
    metadata: FieldMetadata
    date: Date
    text: str


class BraggingRights(object):
    """
    :metadata: Metadata about the bragging rights.
    :value: The bragging rights; for example, climbed mount everest
    """
    metadata: FieldMetadata
    value: str


class ContactGroupMembership(object):
    """
    :contactGroupId: (depreciated) Output only. The contact group ID for the contact group membership.
    :contactGroupResourceName: The resource name for the contact group, assigned by the server. An ASCII
        string, in the form of contactGroups/{contactGroupId}. Only contactGroupResourceName can be used for
        modifying memberships. Any contact group membership can be removed, but only user group or "myContacts"
        or "starred" system groups memberships can be added. A contact must always have at least one contact
        group membership.
    """
    contactGroupId: str
    contactGroupResourceName: str


class ClientData(object):
    metadata: FileMetadata
    key: str
    value: str


class CoverPhoto(object):
    metadata: FieldMetadata
    url: str
    default: bool


class CalendarUrl(object):
    """
    :formattedType: Output only. The type of the calendar URL translated and formatted in the
        viewer's account locale or the Accept-Language HTTP header locale.
    """
    metadata: FieldMetadata
    url: str
    type: str
    formattedType: str


class DomainMembership(object):
    """
    :inViewerDomain: True if the person is in the viewer's Google Workspace domain.
    """
    inViewerDomain: bool


class EmailAddress(object):
    metadata: FieldMetadata
    value: str
    type: str
    formattedType: str
    displayName: str


class Event(object):
    metadata: FieldMetadata
    date: Date
    type: str
    formattedType: str


class ExternalId(object):
    metadata: FieldMetadata
    value: str
    type: str
    formattedType: str


class FileAs(object):
    metadata: FieldMetadata
    value: str


class Gender(object):
    metadata: FieldMetadata
    value: str
    formattedValue: str
    addressMeAs: str


class ImClient(object):
    metadata: FieldMetadata
    username: str
    type: str
    formattedType: str
    protocol: str
    formattedProtocol: str


class Interest(object):
    metadata: FieldMetadata
    value: str


class Locale(object):
    metadata: FieldMetadata
    value: str


class Location(object):
    metadata: {
        FieldMetadata
    }
    value: str
    type: str
    current: bool
    buildingId: str
    floor: str
    floorSection: str
    deskCode: str


class Membership(object):
    """
    :metadata: Metadata about the membership.
    :contactGroupMembership: The contact group membership.
    :domainMembership: Output only. The domain membership.
    """
    metadata: {
        FieldMetadata
    }
    contactGroupMembership: {
        ContactGroupMembership
    }
    domainMembership: {
        DomainMembership
    }


class MiscKeyword(object):
    metadata: {
        FieldMetadata
    }
    value: str
    type: KeywordType
    formattedType: str


class Name(object):
    metadata: {
        FieldMetadata
    }
    displayName: str
    displayNameLastFirst: str
    unstructuredName: str
    familyName: str
    givenName: str
    middleName: str
    honorificPrefix: str
    honorificSuffix: str
    phoneticFullName: str
    phoneticFamilyName: str
    phoneticGivenName: str
    phoneticMiddleName: str
    phoneticHonorificPrefix: str
    phoneticHonorificSuffix: str


class Nickname(object):
    metadata: {
        FieldMetadata
    }
    value: str
    type: NicknameType


class Occupation(object):
    metadata: {
        FieldMetadata
    }
    value: str


class Organization(object):
    metadata: {
        FieldMetadata
    }
    type: str
    formattedType: str
    startDate: {
        Date
    }
    endDate: {
        Date
    }
    current: bool
    name: str
    phoneticName: str
    department: str
    title: str
    jobDescription: str
    symbol: str
    domain: str
    location: str
    costCenter: str
    fullTimeEquivalentMillipercent: int


class PersonMetadata(object):
    soures: [Source]
    previousResourceNames: [str]
    linkedPeopleResourceNames: [str]
    deleted: bool
    objectType: ObjectType


class PhoneNumber(object):
    metadata: {
        FieldMetadata
    }
    value: str
    canonicalForm: str
    type: str
    formattedType: str


class Photo(object):
    metadata: {
        FieldMetadata
    }
    url: str
    default: bool


class Relation(object):
    metadata: {
        FieldMetadata
    }
    person: str
    type: str
    formattedType: str


class RelationshipInterest(object):
    log.warn(DeprecationWarning)

    metadata: {
        FieldMetadata
    }
    value: str
    formattedValue: str


class RelationshipStatus(object):
    log.warn(DeprecationWarning)

    metadata: {
        FieldMetadata
    }
    value: str
    formattedValue: str


class Residence(object):
    log.warn(DeprecationWarning)

    metadata: {
        FieldMetadata
    }
    value: str
    current: bool


class SipAddress(object):
    metadata: {
        FieldMetadata
    }
    value: str
    type: str
    formattedType: str


class Skill(object):
    metadata: {
        FieldMetadata
    }
    value: str


class Tagline(object):
    log.warn(DeprecationWarning)

    metadata: {
        FieldMetadata
    }
    value: str


class Url(object):
    metadata: {
        FieldMetadata
    }
    value: str
    type: str
    formattedType: str


class UserDefined(object):
    metadata: {
        FieldMetadata
    }
    key: str
    value: str


class Person(object):
    resourceName: str
    etag: str
    metadata: PersonMetadata
    addresses: [Address]
    ageRange: AgeRange
    ageRanges: [AgeRangeType]
    biographies: [Biography]
    birthdays: [Birthday]
    braggingRights: [BraggingRights]
    calendarUrls: [CalendarUrl]
    clientData: [ClientData]
    coverPhotos: [CoverPhoto]
    emailAddresses: [EmailAddress]
    events: [Event]
    externalIds: [ExternalId]
    fileAses: [FileAs]
    genders: [Gender]
    imClients: [ImClient]
    interests: [Interest]
    locales: [Locale]
    locations: [Location]
    memberships: [Membership]
    miscKeywords: [MiscKeyword]
    names: [Name]
    nicknames: [Nickname]
    occupations: [Occupation]
    organizations: [Organization]
    phoneNumbers: [PhoneNumber]
    photos: [Photo]
    relations: [Relation]
    relationshipInterests: [RelationshipInterest]
    relationshipStatuses: [RelationshipStatus]
    residences: [Residence]
    sipAddresses: [SipAddress]
    skills: [Skill]
    taglines: [Tagline]
    urls: [Url]
    userDefined: [UserDefined]

    def __init__(self, contact: dict):
        """People Resource"""
        self.__dict__.update(contact)

    def __repr__(self):
        return f'{[x for x in self.get_names() for x in x.values() if isinstance(x, str)][-1]}'

    def to_dict(self):
        return dict(self.__dict__)

    def get_biographies(self):
        return self.__dict__.get('biographies', [])

    def get_memberships(self):
        return self.__dict__.get('memberships', [])

    def get_names(self):
        return self.__dict__.get('names', [])

    def get_phoneNumbers(self):
        return self.__dict__.get('phoneNumbers', [])



import unittest
import hashlib

from automon.integrations.minio.client import MinioClient

c = MinioClient()


class ClientTest(unittest.TestCase):

    def test_isConnected(self):
        if c.isConnected():
            self.assertTrue(c.isConnected())
        else:
            self.assertFalse(c.isConnected())

    def test_clear_bucket(self):
        if c.isConnected():
            bucket = c.make_bucket('AAAAAA')
            if c.list_objects(bucket):
                self.assertTrue(c.remove_objects(bucket))
            else:
                self.assertFalse(c.remove_objects(bucket))


if __name__ == '__main__':
    unittest.main()



import os

from automon.log import Logging
from automon.helpers import Run
from automon.helpers.dates import Dates

from .config import NmapConfig
from .output import NmapResult


class Nmap(object):
    def __init__(self, command: str = None, config: NmapConfig = None, *args, **kwargs):
        self._log = Logging(name=Nmap.__name__, level=Logging.INFO)
        self._runner = Run()

        self.config = config or NmapConfig()
        self.output_file = f'nmap-{Dates.filename_timestamp()}.xml'

        self.result = None
        self.command = str()
        self.error = bytes()

        self._stdout = None
        self._stderr = None
        self._returncode = None

        if command:
            self.run(command=command, *args, **kwargs)

    def __repr__(self):
        if self.result:
            return f'{self.command} ({round(len(self.result) / 1024, 2)} Kb)'
        if self.command:
            return f'{self.command}'
        if self.isReady():
            return f'Ready to scan'
        return f'Not able to scan'

    def __len__(self):
        if self.result:
            return len(self.result)
        return 0

    def isReady(self):
        return self.config.isReady()

    def nmap(self, command: str, **kwargs) -> bool:
        return self.run(command=command, **kwargs)

    def pretty(self):
        return print(self._runner.stdout.decode())

    def run(self, command: str, output: bool = True, cleanup: bool = True, **kwargs) -> bool:

        if not self.isReady():
            self._log.error(enable_traceback=False, msg=f'nmap not found')
            return False

        nmap_command = f'{self.config.nmap} '

        if output:
            nmap_output = f'-oX {self.output_file}'
            nmap_command += f'{nmap_output} '

        nmap_command += f'{command}'

        self._log.info(f'running {nmap_command}')
        self._runner.run(nmap_command, **kwargs)
        self._log.debug(f'finished')

        self.command = nmap_command
        self._stdout = self._runner.stdout
        self._stderr = self._runner.stderr
        self._returncode = self._runner.returncode

        self.error = self._stderr

        if output:
            self.result = NmapResult(file=self.output_file, **kwargs)

            if cleanup:
                os.remove(self.output_file)
                self._log.info(f'deleted {self.output_file}')

        if self._stderr:
            self._log.error(enable_traceback=False, msg=f'{self._stderr.decode()}')
            return False

        return True

    def scan(self, command: str, **kwargs) -> bool:
        return self.run(command=command, **kwargs)



import json
import functools

from automon import Logging
from automon.integrations.requests import Requests

from .rest import Urls
from .artifact import Artifact
from .container import Container
from .config import SplunkSoarConfig
from .results import ResultsContainer

log = Logging(name='SplunkSoarClient', level=Logging.DEBUG)
Logging(name='RequestsClient', level=Logging.DEBUG)


class SplunkSoarClient:
    def __init__(self, host: str = None,
                 user: str = None,
                 password: str = None,
                 config: SplunkSoarConfig = None):
        """Splunk SOAR Client"""

        self.config = config or SplunkSoarConfig(host=host, user=user, password=password)
        self.client = Requests(headers=self.config.headers)

        self.action_run = None
        self.app = None
        self.app_run = None
        self.artifacts = None
        self.asset = None
        self.cluster_node = None
        self.containers = None
        self.playbook_run = None

    def __repr__(self) -> str:
        return f'{self.__dict__}'

    def _content(self) -> bytes:
        """get result"""
        return self.client.results.content

    def _content_dict(self) -> dict:
        """convert request.content to dict"""
        return json.loads(self._content())

    def _get(self, url: str) -> bool:
        """send get request"""
        return self.client.get(url=url, headers=self.client.headers)

    def _isConnected(func):
        @functools.wraps(func)
        def wrapper(self, *args, **kwargs):
            if self.config.isReady():
                return func(self, *args, **kwargs)
            return False

        return wrapper

    def _delete(self, url: str) -> bool:
        """send get request"""
        return self.client.delete(url=url, headers=self.client.headers)

    def _post(self, url: str, data: dict) -> bool:
        """send post request"""
        return self.client.post(url=url, headers=self.client.headers, data=data)

    @_isConnected
    def create_artifact(
            self,
            container_id,
            cef=None,
            cef_types=None,
            data=None,
            description=None,
            end_time=None,
            ingest_app_id=None,
            kill_chain=None,
            label=None,
            name=None,
            owner_id=None,
            run_automation=None,
            severity=None,
            source_data_identifier=None,
            start_time=None,
            tags=None,
            type=None,
            *args, **kwargs) -> Artifact:
        """Create artifact"""

        artifact = Artifact(
            dict(cef=cef,
                 cef_types=cef_types,
                 container_id=container_id,
                 data=data,
                 description=description,
                 end_time=end_time,
                 ingest_app_id=ingest_app_id,
                 kill_chain=kill_chain,
                 label=label,
                 name=name,
                 owner_id=owner_id,
                 run_automation=run_automation,
                 severity=severity,
                 source_data_identifier=source_data_identifier,
                 start_time=start_time,
                 tags=tags,
                 type=type)
        )

        if self._post(Urls().artifact(*args, **kwargs), data=artifact.to_json()):
            if self.client.results.status_code == 200:
                id = self.client.to_dict()['id']
                log.info(f'artifact created. {artifact} {self.client.to_dict()}')
                return self.list_artifact(artifact_id=id)
            else:
                existing_artifact_id = self.client.to_dict()['existing_artifact_id']
                log.info(f'artifact exists. {artifact} {self.client.to_dict()}')
                return self.list_artifact(artifact_id=existing_artifact_id)

        log.error(f'create artifact. {self.client.to_dict()}', enable_traceback=False)
        return False

    @_isConnected
    def create_container(
            self,
            label,
            name,
            artifacts=None,
            asset_id=None,
            close_time=None,
            custom_fields=None,
            data=None,
            description=None,
            due_time=None,
            end_time=None,
            ingest_app_id=None,
            kill_chain=None,
            owner_id=None,
            role_id=None,
            run_automation=None,
            sensitivity=None,
            severity=None,
            source_data_identifier=None,
            start_time=None,
            open_time=None,
            status=None,
            tags=None,
            tenant_id=None,
            container_type=None,
            template_id=None,
            authorized_users=None,
            *args, **kwargs) -> Container:
        """Create container"""

        container = Container(
            dict(label=label,
                 name=name,
                 artifacts=artifacts,
                 asset_id=asset_id,
                 close_time=close_time,
                 custom_fields=custom_fields,
                 data=data,
                 description=description,
                 due_time=due_time,
                 end_time=end_time,
                 ingest_app_id=ingest_app_id,
                 kill_chain=kill_chain,
                 owner_id=owner_id,
                 role_id=role_id,
                 run_automation=run_automation,
                 sensitivity=sensitivity,
                 severity=severity,
                 source_data_identifier=source_data_identifier,
                 start_time=start_time,
                 open_time=open_time,
                 status=status,
                 tags=tags,
                 tenant_id=tenant_id,
                 container_type=container_type,
                 template_id=template_id,
                 authorized_users=authorized_users)
        )

        if self._post(Urls().container(*args, **kwargs), data=container.to_json()):
            if self.client.results.status_code == 200:
                id = self.client.to_dict()['id']
                log.info(f'container created. {container} {self.client.to_dict()}')
                return self.get_container(container_id=id)
        log.error(f'create container. {self.client.to_dict()}', enable_traceback=False)
        return False

    @_isConnected
    def delete_container(self, container_id, *args, **kwargs):
        """Delete containers"""
        assert isinstance(container_id, int)

        if self._delete(Urls().container(identifier=container_id, *args, **kwargs)):
            if self.client.results.status_code == 200:
                log.info(f'container deleted: {container_id}')
                return True
        log.error(f'delete container: {container_id}. {self.client.to_dict()}', enable_traceback=False)
        return False

    def isConnected(self) -> bool:
        """check if client can connect"""
        if self.config.isReady():
            if self._get(Urls().container()):
                log.info(f'client connected '
                         f'{self.config.host} '
                         f'[{self.client.results.status_code}] ')
                return True

        else:
            log.warn(f'client not connected')
        return False

    @_isConnected
    def list_artifact(self, artifact_id: int = None, **kwargs) -> bool:
        """list action run"""
        if self._get(Urls().artifact(identifier=artifact_id, **kwargs)):
            request = self._content_dict()
            if artifact_id:
                return Artifact(request)
            artifacts = [Artifact(a) for a in request['data']]
            return artifacts
        return []

    @_isConnected
    def list_action_run(self, **kwargs) -> bool:
        """list action run"""
        if self._get(Urls().action_run(**kwargs)):
            self.containers = self._content_dict()
            return True
        return False

    @_isConnected
    def list_app(self, **kwargs) -> bool:
        """list app"""
        if self._get(Urls().app(**kwargs)):
            self.app = self._content_dict()
            return True
        return False

    @_isConnected
    def list_app_run(self, **kwargs) -> bool:
        """list app run"""
        if self._get(Urls().app_run(**kwargs)):
            self.app_run = self._content_dict()
            return True
        return False

    @_isConnected
    def list_artifacts(self, **kwargs) -> bool:
        """list artifacts"""
        if self._get(Urls().artifact(**kwargs)):
            self.artifacts = self._content_dict()
            return True
        return False

    @_isConnected
    def list_asset(self, **kwargs) -> bool:
        """list asset"""
        if self._get(Urls().asset(**kwargs)):
            self.asset = self._content_dict()
            return True
        return False

    @_isConnected
    def get_container(self, container_id: int = None, **kwargs) -> Container:
        if self._get(Urls().container(identifier=container_id, **kwargs)):
            request = self._content_dict()
            container = Container(request)
            log.info(f'get container: {container}')
            return container

        log.error(f'container not found: {container_id}', enable_traceback=False)
        return False

    @_isConnected
    def list_containers(self,
                        page: int = None,
                        page_size: int = 1000,
                        *args, **kwargs) -> ResultsContainer:
        """list containers"""

        url = Urls().container(page=page, page_size=page_size, *args, **kwargs)
        if self._get(url):
            request = self._content_dict()
            containers = ResultsContainer(request)
            log.info(f'list containers: {len(containers.data)}')
            return containers
        log.error(f'no containers', enable_traceback=False)
        return ResultsContainer()

    @_isConnected
    def list_containers_generator(self, page: int = 0, page_size: int = None, **kwargs) -> [Container]:
        """Generator for paging through containers"""

        page = page

        while True:
            request = self.list_containers(page=page, page_size=page_size)
            if request.data:
                containers = request.data
                num_pages = request.num_pages
                log.info(f'{page}/{num_pages} ({round(page / num_pages * 100, 2)}%)')

                if page > num_pages:
                    log.info(f'list container finished')
                    return True

                yield containers
                page += 1

            elif request.data == []:
                log.info(f'{page}/{num_pages} ({round(page / num_pages * 100, 2)}%)')
                log.info(f'list container finished. {request}')
                return True

            elif request.data is None:
                log.error(f'list container failed', enable_traceback=True)
                return False

            else:
                log.info(f'no containers. {request}')
                return True

        return False

    @_isConnected
    def list_cluster_node(self, **kwargs) -> bool:
        """list cluster node"""
        if self._get(Urls().cluster_node(**kwargs)):
            self.cluster_node = self._content_dict()
            return True
        return False

    @_isConnected
    def list_playbook_run(self, **kwargs) -> bool:
        """list cluster node"""
        if self._get(Urls().playbook_run(**kwargs)):
            self.playbook_run = self._content_dict()
            return True
        return False

    @_isConnected
    def list_vault(self, identifier=None, **kwargs) -> bool:
        """list cluster node"""
        if self._get(Urls().vault(identifier=identifier, **kwargs)):
            self.playbook_run = self._content_dict()
            return True
        return False



import os
import hashlib

from flask import Flask


class FlaskConfig(Flask):

    @staticmethod
    def javascript_compatibility(app):
        """Javascript expression compatibility

        Required to work with passing args to a flask page
        """

        jinja_options = app.jinja_options.copy()

        jinja_options.update(dict(
            block_start_string='<%',
            block_end_string='%>',
            variable_start_string='%%',
            variable_end_string='%%',
            comment_start_string='<#',
            comment_end_string='#>'
        ))

        return jinja_options

    @staticmethod
    def hash_key(blob):
        """Make a hash key"""
        return hashlib.md5(str(blob).encode()).digest().hex()

    @staticmethod
    def new_secret_key():
        """Create new session key"""
        return os.urandom(16)



from selenium.webdriver.common.action_chains import ActionChains

from automon.log import Logging

log = Logging(name='selenium', level=Logging.INFO)


class SeleniumActions:
    @staticmethod
    def click(browser, xpath):
        """Given an xpath, it will click it

        :param browser: selenium browser
        :param xpath: chrome xpath
        :return:
        """
        element = browser.find_element_by_xpath(xpath)
        return element.click()

    @staticmethod
    def type(browser, keys):
        """Given a browser and a list of keys to perform

        :param browser: browser
        :param keys: list of keys
        :return: perform list of keys
        """
        actions = ActionChains(browser)
        for key in keys:
            actions.send_keys(key)

        return actions.perform()



import socket
from urllib.parse import urlparse

from automon.log import Logging

log = Logging(name='Networking', level=Logging.INFO)


class Networking:

    @staticmethod
    def check_connection(url, timeout: int = 1):
        endpoint = urlparse(url)

        if not endpoint.hostname:
            endpoint = urlparse(f'x://{url}')

        host = endpoint.hostname
        port = endpoint.port

        try:
            s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            s.settimeout(timeout)
            s.connect((host, port))
            s.close()
            log.debug(f'SUCCESS {url}')
            return True
        except Exception as e:
            log.error(f'FAILED {url} {e}', enable_traceback=False)
            return False

    @staticmethod
    def urlparse(url):
        return urlparse(url)



import logging
import traceback

from automon.helpers import Dates
from automon.helpers.markdown import Chat, Format

TEST = 5
DEBUG = logging.DEBUG
INFO = logging.INFO
WARN = logging.WARN
ERROR = logging.ERROR
CRITICAL = logging.CRITICAL
NOTSET = logging.NOTSET

logging.getLogger('logger').setLevel(CRITICAL)


class Callback(object):

    def __init__(self, callbacks: list):
        """Log to callbacks
        """

        self.log = Logging(name=Callback.__name__, level=Logging.DEBUG)
        self.callbacks = callbacks

    def call(self, type: str, msg: str, *args, **kwargs) -> True:
        for call in self.callbacks:
            if type == 'info':
                call.info(msg, *args, **kwargs)

            elif type == 'debug':
                call.debug(msg, *args, **kwargs)

            elif type == 'error':
                call.error(msg, *args, **kwargs)

            elif type == 'warn' or type == 'warning':
                call.warn(msg, *args, **kwargs)

            elif type == 'critical':
                call.critical(msg, *args, **kwargs)

            else:
                call.warn(f'{NotImplemented} {type} {msg}')

        return True

    def info(self, msg: str, *args, **kwargs):
        return self.call(type='info', msg=msg, *args, **kwargs)

    def debug(self, msg: str, *args, **kwargs):
        return self.call(type='debug', msg=msg, *args, **kwargs)

    def error(self, msg: str, *args, **kwargs):
        return self.call(type='error', msg=msg, *args, **kwargs)

    def warn(self, msg: str, *args, **kwargs):
        return self.call(type='warn', msg=msg, *args, **kwargs)

    def warning(self, msg: str, *args, **kwargs):
        return self.call(type='warning', msg=msg, *args, **kwargs)

    def critical(self, msg: str, *args, **kwargs):
        return self.call(type='critical', msg=msg, *args, **kwargs)


class LogStream(object):
    """Allows logging to string
    """

    def __init__(self):
        self.logs = ''

    def write(self, string):
        self.logs += string

    def flush(self):
        pass

    def __repr__(self):
        return self.logs


class Logging(object):
    """Standard logging
    """

    TEST = 5
    DEBUG = DEBUG
    INFO = INFO
    WARN = WARN
    ERROR = ERROR
    CRITICAL = CRITICAL
    NOTSET = NOTSET

    def __init__(self, name: str = __name__,
                 level: int = INFO,
                 file: str = None,
                 encoding: str = 'utf-8',
                 filemode: str = 'a',
                 log_stream: LogStream = False,
                 callbacks: list = None,
                 timestamp: bool = True, *args, **kwargs):

        self.started = Dates.now()

        self.logging = logging.getLogger(name)
        self.logging.setLevel(level)

        self.callbacks = callbacks or []

        spacing = 4

        # logging format
        time = '%(asctime)s'
        levelname = '%(levelname)s'
        logger = '%(name)s'
        filename = '%(filename)s'
        pathname = '%(pathname)s'
        func = '%(funcName)s'
        line = '%(lineno)d'
        module = '%(module)s'
        message = '%(message)s'

        # self.log_format = f'{levelname}\t[{logger}]\t{message}'
        self.log_format = f'{levelname}\t[{logger}]\t[{filename} {func}:L{line}]\t{message}'
        # self.log_format = '%(levelname)s\t%(message)s\t%(name)s'
        # self.log_format = '%(levelname)s\t%(name)s\t%(module)s\t%(message)s'

        if timestamp:
            self.log_format = f'{time}\t{self.log_format}'

        logging.basicConfig(level=level, format=self.log_format, **kwargs)

        if file:
            logging.basicConfig(filename=file, encoding=encoding, filemode=filemode, level=level,
                                format=self.log_format, **kwargs)

        # TODO: need informative logging format
        # TODO: log streaming does not work
        if log_stream:
            self.stream = LogStream() if log_stream else None
            logging.basicConfig(level=level, stream=self.stream)

    def error(self, msg: any = None,
              enable_traceback: bool = True,
              raise_exception: bool = False,
              *args, **kwargs):
        tb = traceback.format_exc()
        # tb = Chat.wrap(tb, Format.codeblock)
        if 'NoneType' not in tb and enable_traceback:
            self.logging.error(tb)

        if msg and not raise_exception:
            self.logging.error(msg, *args, **kwargs)
            Callback(self.callbacks).error(msg, *args, **kwargs)
            return True

        if raise_exception:
            self.logging.error(msg, *args, **kwargs)
            Callback(self.callbacks).error(msg, *args, **kwargs)
            raise Exception(msg, *args, **kwargs)
            return True

    def warning(self, msg: any, *args, **kwargs):
        self.logging.warning(msg, *args, **kwargs)
        Callback(self.callbacks).warning(msg, *args, **kwargs)
        return True

    def warn(self, msg: any, *args, **kwargs):
        self.warning(msg, *args, **kwargs)
        Callback(self.callbacks).warn(msg, *args, **kwargs)
        return True

    def info(self, msg: any, *args, **kwargs):
        self.logging.info(msg, *args, **kwargs)
        Callback(self.callbacks).info(msg, *args, **kwargs)
        return True

    def debug(self, msg: any, *args, **kwargs):
        self.logging.debug(msg, *args, **kwargs)
        Callback(self.callbacks).debug(msg, *args, **kwargs)
        return True

    def critical(self, msg: any, *args, **kwargs):

        tb = traceback.format_exc()
        tb = Chat.wrap(tb, Format.codeblock)
        if 'NoneType' not in tb:
            self.logging.critical(tb)
            Callback(self.callbacks).critical(msg, *args, **kwargs)

        self.logging.critical(msg, *args, **kwargs)
        Callback(self.callbacks).critical(msg, *args, **kwargs)
        return True

    @staticmethod
    def now():
        return Dates.now()

    def uptime(self):
        return Dates.now() - self.started



import re

from swiftclient.service import SwiftService, SwiftError, ClientException

from automon.log import Logging


class SwiftItem(object):
    def __init__(self, item: dict):
        self._log = Logging(SwiftItem.__name__, Logging.DEBUG)
        self._item = item
        self._dict = item

        self.size = int(item.get('bytes', 0))
        self.name = item.get('name')
        self.hash = item.get('hash')
        self.etag = self.hash

        self.content_type = item.get('content_type', None)
        self.last_modified = item.get('last_modified')

        if self.content_type == 'application/directory':
            self._is_directory = True
            self._dir_marker = True
        else:
            self._is_directory = False
            self._dir_marker = False

    def is_directory(self):
        return self._is_directory

    def has_dir_marker(self):
        return self._dir_marker

    def data(self):
        return self._item

    def filter(self, regex: str) -> bool:
        regex = str(regex)
        if re.search(regex, self.name):
            return True
        else:
            return False

    def __repr__(self):
        return f'{self.name} [size: {self.size}] [etag: {self.etag}]'


class SwiftPage(SwiftService):
    def __init__(self, page: dict) -> SwiftService.list:
        self._log = Logging(SwiftPage.__name__, Logging.ERROR)

        self._page = page

        self.action = self._page.get('action')
        self.container = self._page.get('container')
        self.prefix = self._page.get('prefix')
        self.success = self._page.get('success')
        self.marker = self._page.get('marker')
        self.error = self._page.get('error')
        self.traceback = self._page.get('traceback')
        self.error_timestamp = self._page.get('error_timestamp')

        if self.error == ClientException:
            self._log.error(f'{SwiftPage.__name__} {self.success} {self.error}')

        if self.success:
            self.listing = self._page.get('listing')
            self._log.debug(f'{SwiftPage.__name__} {self.success} {self.listing}')
        else:
            self.listing = []

    def _dict(self):
        return {
            'action': self.action,
            'container': self.container,
            'prefix': self.prefix,
            'success': self.success,
            'marker': self.marker,
            'error': self.error,
            'traceback': self.traceback,
            'error_timestamp': self.error_timestamp,
            'listing': self.listing
        }

    def _error_handler(self):
        if not self.success and isinstance(self.error, Exception):
            self._log.error(f'{SwiftError(self._page)}')

    def list_gen(self) -> object or SwiftItem:
        if self.success:
            for LIST in self.listing:
                yield SwiftItem(LIST)
        else:
            self._error_handler()

    def __repr__(self):
        return f'{self._page}'


class SwiftList(SwiftService):
    def __init__(self, container: str) -> SwiftService.list:
        """
        always request a new SwiftService object
        see documentation
        """

        self._log = Logging(SwiftList.__name__, Logging.DEBUG)
        self.container = container

    def list_gen(self) -> object or SwiftPage:
        with SwiftService() as swift:
            try:
                list_parts_gen = swift.list(container=self.container)

                # TODO: need to check if gi_running is True when connected, or always False
                if list_parts_gen.gi_running:
                    for page in list_parts_gen:
                        if page["success"]:
                            yield SwiftPage(page)
                        else:
                            self._log.error(f'{page["error"]}')

            except Exception as e:
                self._log.error(f'page failed, {e}')
